local bump       = require 'bump'
local bump_debug = require 'bump_debug'

local instructions = [[
  bump.lua simple demo

    arrows: move
    tab: toggle debug info
    delete: run garbage collector
]]

local cols_len = 0 -- how many collisions are happening

-- World creation
local world = bump.newWorld()


-- Message/debug functions
local function drawMessage()
  local msg = instructions:format(tostring(shouldDrawDebug))
  love.graphics.setColor(255, 255, 255)
  love.graphics.print(msg, 550, 10)
end

local function drawDebug()
  bump_debug.draw(world)

  local statistics = ("fps: %d, mem: %dKB, collisions: %d, items: %d"):format(love.timer.getFPS(), collectgarbage("count"), cols_len, world:countItems())
  love.graphics.setColor(255, 255, 255)
  love.graphics.printf(statistics, 0, 580, 790, 'right')
end

local consoleBuffer = {}
local consoleBufferSize = 15
for i=1,consoleBufferSize do consoleBuffer[i] = "" end
local function consolePrint(msg)
  table.remove(consoleBuffer,1)
  consoleBuffer[consoleBufferSize] = msg
end

local function drawConsole()
  local str = table.concat(consoleBuffer, "\n")
  for i=1,consoleBufferSize do
    love.graphics.setColor(255,255,255, i*255/consoleBufferSize)
    love.graphics.printf(consoleBuffer[i], 10, 580-(consoleBufferSize - i)*12, 790, "left")
  end
end

-- helper function
local function drawBox(box, r,g,b)
  love.graphics.setColor(r,g,b,70)
  love.graphics.rectangle("fill", box.x, box.y, box.w, box.h)
  love.graphics.setColor(r,g,b)
  love.graphics.rectangle("line", box.x, box.y, box.w, box.h)
end



-- Player functions
local player = {
	x=50,
	y=50,
	w=20,
	h=20,
	
	canJump = 2,
	speed = 200,
	jump = 300,
	grav = 10,
	vY = 0
}

local function updatePlayer(dt)
	local speed = player.speed

	local dx, dy = 0, 0
	if love.keyboard.isDown('right') then
		dx = speed
	elseif love.keyboard.isDown('left') then
		dx = -speed
	end
	
	dy = dy + player.vY
	player.vY = player.vY + player.grav

	if dx ~= 0 or dy ~= 0 then
		local cols
		player.x, player.y, cols, cols_len = world:move(player, player.x + dx*dt, player.y + dy*dt)
	end
  
	local _, _, cols = world:check(player, player.x, player.y + 1)
	if #cols > 0 then player.vY = player.grav; player.canJump = 2 end
end

local function drawPlayer()
	drawBox(player, 0, 255, 0)
end

-- Block functions

local blocks = {}

local function addBlock(x,y,w,h)
	local block = {x=x,y=y,w=w,h=h}
	blocks[#blocks+1] = block
	world:add(block, x,y,w,h)
end

local function drawBlocks()
	for _,block in ipairs(blocks) do
		drawBox(block, 255,0,0)
	end
end




-- Main LÖVE functions
local x0, y0 = 32, 600-32
local a = math.rad(-30)
ramp = {
	x0 = x0,
	y0 = y0,
	x = x0,
	y = y0,
	a = a,
	w = player.w,
	h = player.h,
}

function love.load()
	world:add(player, player.x, player.y, player.w, player.h)

	addBlock(0,       0,     800, 32)
	addBlock(0,      32,      32, 600-32*2)
	addBlock(800-32, 32,      32, 600-32*2)
	addBlock(0,      600-32, 800, 32)
  
	--world:add(ramp, ramp.x, ramp.y, player.w, player.h)
end

function love.update(dt)
	cols_len = 0
	updatePlayer(dt)
  
	-- y - y0 = tan(a) * ( x - x0 )
	--world:update(ramp, player.x, math.sin(ramp.a) * (ramp.x - ramp.x0) + (ramp.y0))
	ramp.x = player.x
	ramp.y = math.tan(ramp.a) * (ramp.x - ramp.x0) + (ramp.y0)
  
	local items, len = world:queryRect(ramp.x, ramp.y, ramp.w, ramp.h)
	for i=1,#items do
		if items[i].x == player.x and items[i].y == player.y then
			player.y = ramp.y - player.h
			player.canJump = 2
			player.vY = player.grav*10;
		end
	end
end

function love.draw()
	love.graphics.setColor(0, 0, 255)
	love.graphics.line(ramp.x0, ramp.y0, love.graphics.getWidth(), math.tan(ramp.a) * (love.graphics.getWidth() - ramp.x0) + (ramp.y0))
	drawBlocks()
	drawPlayer()
	
	if shouldDrawDebug then
		drawBox(ramp, 0, 0, 255)
		drawDebug()
		drawConsole()
	end
	
	love.graphics.print(math.tan(ramp.a) * (love.graphics.getWidth() - ramp.x0) + (ramp.y0))
	drawMessage()
end

-- Non-player keypresses
function love.keypressed(k)
	if k=="escape" then love.event.quit() end
	if k=="tab"    then shouldDrawDebug = not shouldDrawDebug end
	if k=="delete" then collectgarbage("collect") end
	
	if k == "space" then
		if player.canJump > 0 then player.vY = -player.jump; player.canJump = player.canJump - 1 end
	end
end
